import paddle
import paddle.nn as nn
from paddleslim.lc.quantizers import FP4Quantizer
from .linear import Linear4bit


class FP4Linear(Linear4bit):
    """
    FP4Linear是一个线性层，使用FP4量化进行权重量化。
    Args:
        linear: 线性层。
        block_size (int): 量化块的大小，默认为64。
        use_double_quant (bool): 是否使用双量化，默认为False。
    """

    def __init__(
            self,
            linear,
            block_size=64,
            use_double_quant=False, ):
        """
        """
        super(FP4Linear, self).__init__(linear, quant_type="fp4")
        self.block_size = block_size
        self.double_quant = use_double_quant
        self.quantizer = FP4Quantizer(block_size, self.double_quant)

    def quantize(self):
        """
        对权重进行量化。
        """
        quantized_weight = self.quantizer.quantize(self.linear.weight).reshape(
            [self.out_features // 2, self.in_features])
        del self.linear.weight
        paddle.assign(quantized_weight, self.quant_weight)

    def forward(self, x):
        """
        前向传播。
        Args:
            x (paddle.Tensor): 输入张量。
        Returns:
            paddle.Tensor: 经过线性层后的输出张量。
        """
        return self.quantizer.matmul(x, self.quant_weight.reshape([-1, 1]), self.linear.bias)