# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import paddle

class BaseQuantizer():
    def quantize(self, x):
        """
        将输入的Tensor进行量化处理
        
        Args:
            x (paddle.Tensor): 输入的Paddle张量，需要进行量化处理
        
        Returns:
            paddle.Tensor: 返回经过量化处理后的张量
        
        Raises:
            NotImplementedError: 如果当前方法未被实现则抛出此异常
        """
        raise NotImplementedError()

    def dequantize(self, x):
        """
        Args:
            x (paddle.Tensor): Input Tensor of shape [N, C]
        
        Returns:
            paddle.Tensor: Dequantized output tensor of shape [N, C].
                The data type is int8 or uint8 depending on the input quantization parameters.
        
        Raises:
            NotImplementedError: If not implemented by subclass.
        
        """
        raise NotImplementedError()

    def matmul(self, x, y, bias=None):
        """
        Args:
            x (paddle.Tensor): The input tensor with shape [batch_size, seq_len].
            y (paddle.Tensor): The weight tensor with shape [seq_len, hidden_size].
            bias (paddle.Tensor): The bias tensor with shape [hidden_size]
        
        Returns:
            output (paddle.Tensor):
                A Tensor of shape [batch_size, hidden_size], which is calculated by the linear transformation between
                `x` and `y`.
        
        Raises:
            NotImplementedError: If the function is not implemented yet.
        
        """
        raise NotImplementedError()
